@@ -17,7 +17,7 @@ import re
 import math
 from read_roi import read_roi_zip
 
-__version__ = "0.25.1"
+__version__ = "0.27.6"
 
 # def get_one_annotation(viewstate):
 #     titles_list = viewstate.xpath(".//title/text()")
@@ -133,6 +133,11 @@ def ndpa_to_json(path):
 
 
 def get_imsize_from_imagej_roi(rois):
+    """
+    Get size from ImageJ annotation. There have to be rectangle over the whole image.
+    :param rois:
+    :return:
+    """
     width = 0
     height = 0
     rect = False
@@ -149,15 +154,18 @@ def get_imsize_from_imagej_roi(rois):
                 height = heighti
 
             rect = True
-        if not rect:
-            raise Exception(
-                "There should be rectangle in ROI file to define image size."
-            )
+    if not rect:
+        raise Exception(
+            "There should be rectangle in ROI file to define image size."
+        )
     return np.array([height, width])
 
 
 def read_annotations_imagej(path, slide_size) -> list:
     """
+    Read annotation from ImageJ. It is stored in `.roi.zip` file. There should be one rectangle annotation over
+    whole image to allow get size of the image used to produce the ROI.
+    All polygon-type annotations should contain color code in name like #FF0000 for red.
     :param path: Path to original image file. The ROI file name is derived from by adding .roi.zip
     :param slide_size:
     :return:
@@ -165,7 +173,7 @@ def read_annotations_imagej(path, slide_size) -> list:
     # import io3d
     # import numpy as np
     fn = path + ".roi.zip"
-    logger.debug("Looking for ROI file {fn}")
+    logger.debug(f"Looking for ROI file {fn}")
     anns = []
     if op.exists(fn):
         # def read_annotations_imagej(, slide_size=slide_size):
@@ -265,11 +273,14 @@ def plot_annotations(
         x = np.asarray(annotation[x_key]) * factor[0]
         y = np.asarray(annotation[y_key]) * factor[1]
         # plt.hold(True)
-        plt.plot(x, y, c=annotation["color"])
-        if show_id:
-            plt.text(
-                np.min(x), np.min(y), str(i), c=annotation["color"], fontsize="x-small"
-            )
+        if len(x) == 0 or len(y) == 0:
+            logger.debug(f"Annotation id={i} has zero length")
+        else:
+            plt.plot(x, y, c=annotation["color"])
+            if show_id:
+                plt.text(
+                    np.min(x), np.min(y), str(i), c=annotation["color"], fontsize="x-small"
+                )
 
 
 def adjust_xy_to_image_view(imsl, x_px, y_px, center, level, size):
