@@ -35,6 +35,7 @@ from matplotlib.path import Path as mplPath
 from mpl_toolkits.axes_grid1 import make_axes_locatable
 from PIL import Image
 from PIL.TiffTags import TAGS
+import traceback
 
 annotationID = Union[int, str]
 annotationIDs = List[annotationID]
@@ -44,20 +45,20 @@ annotationIDs = List[annotationID]
 
 
 def import_openslide():
-
-    pth = op.expanduser(r"~\Downloads\openslide-win64-20171122\bin")
-    dll_list = glob.glob(op.join(pth, "*.dll"))
-    if len(dll_list) < 5:
-        print("Trying to download openslide dll files in {}".format(pth))
-        libfixer.libfix()
-    # pth = op.expanduser(r"~\projects\scaffan\devel\knihovny")
-    # pth = op.expanduser(r"~\Miniconda3\envs\lisa36\Library\bin")
-    sys.path.insert(0, pth)
-    orig_PATH = os.environ["PATH"]
-    orig_split = orig_PATH.split(";")
-    if pth not in orig_split:
-        print("add path {}".format(pth))
-    os.environ["PATH"] = pth + ";" + os.environ["PATH"]
+    if os.name == 'nt':
+        pth = op.expanduser(r"~\Downloads\openslide-win64-20171122\bin")
+        dll_list = glob.glob(op.join(pth, "*.dll"))
+        if len(dll_list) < 5:
+            print("Trying to download openslide dll files in {}".format(pth))
+            libfixer.libfix()
+        # pth = op.expanduser(r"~\projects\scaffan\devel\knihovny")
+        # pth = op.expanduser(r"~\Miniconda3\envs\lisa36\Library\bin")
+        sys.path.insert(0, pth)
+        orig_PATH = os.environ["PATH"]
+        orig_split = orig_PATH.split(";")
+        if pth not in orig_split:
+            logger.debug("add path {}".format(pth))
+        os.environ["PATH"] = pth + ";" + os.environ["PATH"]
 
 
 import_openslide()
@@ -357,23 +358,24 @@ class ImageSlide:
             xr = meta_dict["XResolution"]
             logger.debug(f"xr={xr}")
             xres = (xr[0][1] / xr[0][0]) * unit_multiplicator
-            self.parameters.param("Input", "Pixel Size X").setValue(xres)
+            # self.parameters.param("Input", "Pixel Size X").setValue(xres)
             yr = meta_dict["YResolution"]
             logger.debug(f"yr={xr}")
             yres = (yr[0][1] / yr[0][0]) * unit_multiplicator
         except Exception as e:
+            logger.debug(traceback.format_exc())
             logger.warning("Resolution reading failed")
             xres = 1.0
             yres = 1.0
 
         self.dimensions = [meta_dict["ImageLength"][0], meta_dict["ImageWidth"][0]]
         meta_dict["tiff.ResolutionUnit"] = "m"
-        meta_dict["tiff.XResolution"] = xres
-        meta_dict["tiff.YResolution"] = yres
+        meta_dict["tiff.XResolution"] = 1.0 / xres
+        meta_dict["tiff.YResolution"] = 1.0 / yres
         meta_dict["hamamatsu.XOffsetFromSlideCentre"] = 0
         meta_dict["hamamatsu.YOffsetFromSlideCentre"] = 0
-        meta_dict["openslide.level[0].height"] = self.dimensions[0]
-        meta_dict["openslide.level[0].width"] = self.dimensions[1]
+        meta_dict["openslide.level[0].height"] = self.dimensions[1]
+        meta_dict["openslide.level[0].width"] = self.dimensions[0]
         self.properties = meta_dict
 
     #     if self.openslide is not None:
@@ -601,6 +603,7 @@ class AnnotatedImage:
             view = view_level0.to_level(level)
         else:
             view = view_level0
+        # resize annotations
         return view
 
     def get_view(
@@ -956,7 +959,10 @@ class AnnotatedImage:
         )
         im = np.asarray(imcr)
         if as_gray:
-            im = skimage.color.rgb2gray(im)
+            if (len(im.shape) > 2):
+                if im.shape[2] == 4:
+                    im = skimage.color.rgba2rgb(im)
+                im = skimage.color.rgb2gray(im)
             if as_unit8:
                 im = (im * 255).astype(np.uint8)
 
@@ -1400,7 +1406,10 @@ class View:
         # logger.debug(f"imcr dtype: {image1.dtype}, shape: {image1.shape}, min max: [{np.min(image1[:,:,:3])}, {np.max(image1[:,:,:3])}], mean: {np.mean(image1[:,:,:3])}, min max alpha: [{np.min(image1[:,:,3])}, {np.max(image1[:,:,3])}], mean: {np.mean(image1[:,:,3])}")
 
         if as_gray:
-            im = skimage.color.rgb2gray(im)
+            if (len(im.shape) > 2):
+                if im.shape[2] == 4:
+                    im = skimage.color.rgba2rgb(im)
+                im = skimage.color.rgb2gray(im)
 
         if self._is_resized_by_pixelsize:
             pxsz_level, pxunit_level = self.anim.get_pixel_size(level=self.region_level)
@@ -1540,6 +1549,43 @@ class View:
             margin_in_pixels=margin_in_pixels,
         )
 
+    def get_training_labels(self, fill_gaps=False, return_debug_images=False):
+        view = self
+        seg_black = view.get_annotation_raster_by_color(
+            "#000000", raise_exception_if_not_found=False
+        )
+        seg_magenta = view.get_annotation_raster_by_color(
+            "#FF00FF", raise_exception_if_not_found=False
+        )
+        seg_red = view.get_annotation_raster_by_color(
+            "#FF0000", raise_exception_if_not_found=False
+        )
+        # find overlays
+        overlays = (1 * seg_black + 1 * seg_magenta + 1 * seg_red) > 1
+        segmentation = 2 * seg_black + 1 * seg_magenta + 3 * seg_red
+        # remove overlays
+        segmentation[overlays] = 0
+        if not fill_gaps:
+            if return_debug_images:
+                return segmentation, []
+            return segmentation
+        else:
+            from scipy.ndimage import morphology
+            dst, inds = morphology.distance_transform_edt(segmentation==0, return_indices=True)
+            # plt.imshow(dst)
+
+            # fill the gaps
+            filled = segmentation[[*inds]]
+            # plt.imshow(filled, vim=1)
+            # plt.colorbar()
+            if return_debug_images:
+                return filled, [dst]
+            else:
+                return filled
+
+
+
+
 
 class ColorError(Exception):
     pass
