@@ -22,6 +22,8 @@ from sklearn.naive_bayes import GaussianNB
 import numpy as np
 from skimage.feature import peak_local_max
 import skimage.filters
+import sklearn.metrics
+# import skimage.metrics
 from skimage.morphology import disk
 import scipy.ndimage
 import matplotlib.pyplot as plt
@@ -40,6 +42,7 @@ from exsu.report import Report
 from .image import AnnotatedImage
 from . import texture
 from . import whole_slide_seg_unet
+import scaffan
 
 
 class ScanSegmentation:
@@ -49,7 +52,7 @@ class ScanSegmentation:
         pname="Scan Segmentation",
         ptype="bool",
         pvalue=True,
-        ptip="Run analysis of whole slide before all other processing is perfomed.\n"
+        ptip="Run analysis of whole scan before all other processing is perfomed.\n"
         + "If automatic lobulus selection is selected, "
         + "defined number of biggest lobuli are selected for texture analysis.",
     ):
@@ -66,6 +69,18 @@ class ScanSegmentation:
         #     glcm_levels=128
         # )
         self._unet = whole_slide_seg_unet.WholeSlideSegmentationUNet(report=report)
+
+        self._glcm = scaffan.texture.GLCMTextureMeasurement(
+            "whole_slide_glcm",
+            texture_label="ws_part",
+            add_cols_to_report=False,
+            report=report,
+            # tile_size=64,
+            # tile_spacing=32,
+            tile_size=16,
+            tile_spacing=8,
+            report_severity_offset=-30
+        )
         params = [
             # {
             #     "name": "Tile Size",
@@ -75,7 +90,9 @@ class ScanSegmentation:
             {
                 "name": "Working Resolution",
                 "type": "float",
-                "value": 0.00001,  # 0.01 mm
+                # "value": 0.00001,  # 0.01 mm
+                "value": 0.000005,  # 0.005 mm
+                "value": 0.00000355,  # 0.005 mm
                 "suffix": "m",
                 "siPrefix": True,
                 "tip": "Resolution used for segmentation processing. "
@@ -92,14 +109,15 @@ class ScanSegmentation:
             {
                 "name": "Segmentation Method",
                 "type": "list",
-                "values": ["U-Net", "HCTFS"],
-                "value": "U-Net",
+                "values": ["U-Net", "HCTFS", "GLCMTFS"],
+                "value": "GLCMTFS",
+                # "value": "U-Net",
             },
             self._unet.parameters,
             {
-                "name": "HCTFS",
+                "name": "TFS General",
                 "type": "group",
-                "tip": "Hand-crafter Texture Features based Segmentation parameters",
+                "tip": "Texture Features based Segmentation parameters",
                 "expanded": False,
                 "children": [
                     {
@@ -107,9 +125,9 @@ class ScanSegmentation:
                         "type": "bool",
                         "value": False,
                         "tip": "Use annotated image to train classifier.\n"
-                        + "Red area is extra-lobular tissue.\n"
-                        + "Black area is intra-lobular tissue.\n"
-                        + "Magenta area is empty part of the image.\n",
+                        + "Red (#FF0000) area is extra-lobular tissue.\n"
+                        + "Black (#000000) area is intra-lobular tissue.\n"
+                        + "Magenta (#FF00FF) area is empty part of the image.\n",
                     },
                     {
                         "name": "Load Default Classifier",
@@ -152,13 +170,32 @@ class ScanSegmentation:
                 + "It is not used in standard processing. "
                 + "The training labels are: 'unknown', 'background', 'intralobular' and 'interlobular'.",
             },
+            {
+                "name": "Postprocessing",
+                "type": "bool",
+                "value": True,
+                "tip": "Run postprocessing based on intra-lobular smoothing and maximal distance from extra-lobular tissue",
+                "expanded": False,
+                "children": [
+                    {
+                        "name": "Max. Distance",
+                        "type": "float",
+                        "value": 0.001,  # 1 mm
+                        # "value": 0.000005,  # 0.005 mm
+                        # "value": 0.00000355,  # 0.005 mm
+                        "suffix": "m",
+                        "siPrefix": True,
+                        "tip": "Maximal distance from extra-lobular tissue"
+                    },
+                ]
+            },
             {
                 "name": "Lobulus Number",
                 "type": "int",
                 "value": 5,
                 # "suffix": "px",
                 "siPrefix": False,
-                "tip": "Number of lobuluses automatically selected after whole slide segmentation",
+                "tip": "Number of lobuluses automatically selected after whole scan segmentation",
             },
             {
                 "name": "Annotation Radius",
@@ -219,14 +256,9 @@ class ScanSegmentation:
         # QuadraticDiscriminantAnalysis(),
         # GaussianNB(),
         # self.clf = GaussianNB()
-        self.clf = self._clf_object(**self._clf_params)
-        self.clf_fn: Path = Path(Path(__file__).parent / "segmentation_model.pkl")
-        self.clf_default_fn: Path = Path(
-            Path(__file__).parent / "segmentation_model_default.pkl"
-        )
-        if self.clf_fn.exists():
-            logger.debug(f"Reading classifier from {str(self.clf_fn)}")
-            self.clf = joblib.load(self.clf_fn)
+        self.clf = None
+        self.clf_fn = None
+        self.clf_default_fn = None
         self.predicted_tiles = None
         # self.output_label_fn = "label.png"
         # self.output_raster_fn = "image.png"
@@ -240,6 +272,7 @@ class ScanSegmentation:
         self.septum_area_mm = None
         self.sinusoidal_area_mm = None
         self.alternative_get_features = None
+        self.accuracy = None
 
     # def init(self, anim: scim.AnnotatedImage):
     def init(self, view: scim.View):
@@ -256,6 +289,20 @@ class ScanSegmentation:
         self.tile_size = None
         self.ann_biggest_ids = []
         self.make_tiles()
+        self.init_clf()
+
+    def init_clf(self):
+        method = str(self.parameters.param("Segmentation Method").value())
+        logger.debug(f"method={method}")
+        method_str = method.replace(" ", "_")
+        self.clf = self._clf_object(**self._clf_params)
+        self.clf_fn: Path = Path(Path(__file__).parent / f"segmentation_model_{method_str}.pkl")
+        self.clf_default_fn: Path = Path(
+            Path(__file__).parent / f"segmentation_model_default_{method_str}.pkl"
+        )
+        if self.clf_fn.exists():
+            logger.debug(f"Reading classifier from {str(self.clf_fn)}")
+            self.clf = joblib.load(self.clf_fn)
 
     def train_classifier(self, pixels=None, y=None, sample_weight: float = None):
         logger.debug("start training")
@@ -264,11 +311,11 @@ class ScanSegmentation:
             pixels, y = self.prepare_training_pixels()
         if sample_weight is None:
             sample_weight = float(
-                self.parameters.param("HCTFS", "Training Weight").value()
+                self.parameters.param("TFS General", "Training Weight").value()
             )
         sample_weight = [sample_weight] * len(y)
 
-        if bool(self.parameters.param("HCTFS", "Clean Before Training").value()):
+        if bool(self.parameters.param("TFS General", "Clean Before Training").value()):
             self.clf = self._clf_object(**self._clf_params)
             # self.clf = GaussianNB()
             logger.debug(f"cleaning the classifier {self.clf}")
@@ -319,14 +366,33 @@ class ScanSegmentation:
             #             plt.figure()
             #             plt.imshow(ann_raster)
             #             plt.show()
-            img = self._get_features(view_ann)
-            stride = int(self.parameters.param("HCTFS", "Training Stride").value())
+            img = self._get_features(view_ann, annotation_id=id1)
+            stride = int(self.parameters.param("TFS General", "Training Stride").value())
             pixels = img[ann_raster][::stride]
             pixels_list.append(pixels)
         pixels_all = np.concatenate(pixels_list, axis=0)
         return pixels_all
 
-    def _get_features(self, view: View, debug_return=False) -> np.ndarray:
+    def _get_features(self, view: View, debug_return=False, annotation_id=None) -> np.ndarray:
+        method = str(self.parameters.param("Segmentation Method").value())
+        if method == "HCTFS":
+            return self._get_features_hctf(view, debug_return=debug_return)
+        elif method == "GLCMTFS":
+            texture_label_str_id = f"{view.region_location[0]}_{view.region_location[1]}" if annotation_id is None else None
+            self._glcm.set_input_data(
+                view=view, annotation_id=annotation_id, lobulus_segmentation=None,
+                texture_label_str_id=texture_label_str_id
+
+            )
+            self._glcm.run(recalculate_view=False)
+            features = self._glcm.measured_features
+            if debug_return:
+                return features, []
+            else:
+                return features
+        pass
+
+    def _get_features_hctf(self, view: View, debug_return=False) -> np.ndarray:
         """
         Three colors and one gaussian smooth reg channel.
         An alternative features computation can be done by setting callback function
@@ -487,14 +553,14 @@ class ScanSegmentation:
 
     def predict_on_view(self, view):
         smethod = str(self.parameters.param("Segmentation Method").value())
-        if smethod == "HCTFS":
-            return self.predict_on_view_hctfs(view)
+        if smethod in ["HCTFS", "GLCMTFS"]:
+            return self.predict_on_view_tfs(view)
         elif smethod == "U-Net":
             return self._unet.predict_tile(view)
         else:
             raise ValueError(f"Unknown segmentation method '{smethod}'")
 
-    def predict_on_view_hctfs(self, view):
+    def predict_on_view_tfs(self, view):
         image = self._get_features(view)
         fvs = image.reshape(-1, image.shape[2])
         #         print(f"fvs: {fvs[:10]}")
@@ -526,20 +592,7 @@ class ScanSegmentation:
         for view, tile_params in self.tiles:
             # view = self.tiles[ix][iy]
             sl_tl, sl_gl, loc = tile_params
-            seg_black = view.get_annotation_raster_by_color(
-                "#000000", raise_exception_if_not_found=False
-            )
-            seg_magenta = view.get_annotation_raster_by_color(
-                "#FF00FF", raise_exception_if_not_found=False
-            )
-            seg_red = view.get_annotation_raster_by_color(
-                "#FF0000", raise_exception_if_not_found=False
-            )
-            # find overlays
-            overlays = (1 * seg_black + 1 * seg_magenta + 1 * seg_red) > 1
-            segmentation = 2 * seg_black + 1 * seg_magenta + 3 * seg_red
-            # remove overlays
-            segmentation[overlays] = 0
+            segmentation = view.get_training_labels()
             output_image[
                 sl_gl
             ] = segmentation  # .get_region_image(as_gray=as_gray)[sl_x_tl, sl_y_tl, :3]
@@ -548,10 +601,20 @@ class ScanSegmentation:
         self.report.imsave(
             "whole_slide_training_labels.png",
             self.whole_slide_training_labels,
-            level_skimage=20,
+            level_skimage=40,
             level_npz=30,
         )
 
+    def _imsave_full_raster_and_training_labels(self):
+        if (self.full_raster_image is not None) and (self.whole_slide_training_labels is not None):
+            fig = plt.figure()
+            plt.imshow(self.full_raster_image)
+            plt.contour(self.whole_slide_training_labels,
+                        # cmap="Purples"
+                        )
+            self.report.savefig("whole_slide_raster_and_training_labels.png", level=45)
+            plt.close(fig)
+
     def predict_tiles(self):
         if self.tiles is None:
             self.make_tiles()
@@ -630,7 +693,8 @@ class ScanSegmentation:
         logger.debug(f"output_image.shape={output_image.shape}")
         full_image = output_image[self._full_image_crop_slice()]
         self.full_prefilter_image = full_image
-        if str(self.parameters.param("Segmentation Method").value()) == "HCTFS":
+        # if str(self.parameters.param("Segmentation Method").value()) in ["HCTFS", "GLCMTFS"]:
+        if bool(self.parameters.param("Postprocessing").value()):
             self.full_output_image = self._labeling_filtration(full_image)
         else:
             self.full_output_image = full_image
@@ -655,11 +719,20 @@ class ScanSegmentation:
         smooth label 0 and label 1, keep label 2
         """
         logger.debug("labeling filtration")
+        # r_m = float(self.parameters.param("Annotation Radius").value())  # * 1000 # mm
+        resolution_m = float(
+            self.parameters.param("Working Resolution").value()
+        )  # * 1000
+        max_dist_m = float(
+            self.parameters.param("Postprocessing", "Max. Distance").value()
+        )  # * 1000
         tmp_img = full_image.copy()
         tmp_img[full_image == 2] = 1
         import skimage.filters
+        mask = scipy.ndimage.distance_transform_edt(full_image != 2, sampling=resolution_m) < max_dist_m
+        tmp_img[~mask] = 0
 
-        tmp_img = skimage.filters.gaussian(tmp_img.astype(np.float), sigma=4)
+        tmp_img = skimage.filters.gaussian(tmp_img.astype(np.float), sigma=4, preserve_range=False)
 
         tmp_img = (tmp_img > 0.5).astype(np.int)
         tmp_img[full_image == 2] = 2
@@ -793,11 +866,19 @@ class ScanSegmentation:
         img = self.get_raster_image(as_gray=False)
         #         plt.imshow(img)
         self.report.imsave(
-            "slice_raster.png", img.astype(np.uint8), level_skimage=20, level_npz=30
+            "slice_raster.png", img.astype(np.uint8), level_skimage=40, level_npz=30
         )
         fig = plt.figure()
-        plt.imshow(img)
-        self.view.plot_annotations(None)
+        # plt.imshow(img)
+        # view_tmp = self.view
+        # the view is wrong if it is based on whole scan image
+        view_tmp = self.view.to_pixelsize(self.used_pixelsize_mm)
+        plt.imshow(view_tmp.get_raster_image(as_gray=False))
+
+        # logger.debug(f"slice_raster size on pixelsize {self.view.region_pixelsize}, {self.view.annotations}")
+        logger.debug(self.view.region_size_on_level)
+        logger.debug(f"view.region_level={self.view.region_level}")
+        view_tmp.plot_annotations(None)
         # self.view.region_imshow_annotation(i=None)
         # self.view.imshow()
         self.report.savefig("slice_raster_with_annotation.png", level=45)
@@ -819,6 +900,33 @@ class ScanSegmentation:
             }
         )
 
+    def evaluate_labels(self):
+
+        if self.whole_slide_training_labels is not None and self.full_output_image is not None:
+            # labels_true = (self.whole_slide_training_labels - 1).astype(np.int8)
+
+            selection = self.whole_slide_training_labels > 0
+            if np.sum(selection) > 0:
+                accuracy = sklearn.metrics.accuracy_score(
+                    self.whole_slide_training_labels[selection].ravel() - 1,
+                    self.full_output_image[selection].ravel(),
+                    normalize=True
+                )
+                self.report.set_persistent_cols(
+                    {
+                        "Whole Scan Training Labels Accuracy": accuracy
+                    }
+                )
+                self.accuracy = accuracy
+                fig = plt.figure()
+                plt.imshow(self.full_output_image, cmap="Purples")
+                plt.contour(self.whole_slide_training_labels, cmap="Reds")
+                self.report.savefig("segmentation_and_training_labels.png", level=45)
+                plt.close(fig)
+                return accuracy
+        return None
+
+
     def _find_biggest_lobuli(self):
         """
         :param n_max: Number of points. All points are returned if set to negative values.
@@ -846,7 +954,7 @@ class ScanSegmentation:
         # image_max = scipy.ndimage.maximum_filter(dist, size=min_distance, mode="constant")
         # Comparison between image_max and im to find the coordinates of local maxima
         coordinates = peak_local_max(dist, min_distance=min_distance)
-        point_dist = dist[list(zip(*coordinates))]
+        point_dist = dist[tuple(list(zip(*coordinates)))]
         # display(point_dist)
         # max_point_inds = point_dist.argsort()[-n_max:][::-1]
         sorted_point_inds = point_dist.argsort()[::-1]
@@ -881,12 +989,12 @@ class ScanSegmentation:
             "ro",
         )
         plt.axis("off")
+        self.report.savefig(
+            "sinusoidal_tissue_local_centers.png", level=55
+        )
         self.report.savefig_and_show(
             "sinusoidal_tissue_local_centers.pdf", fig, level=55
         )
-        # self.report.savefig_and_show(
-        #     "sinusoidal_tissue_local_centers.png", fig, level=55
-        # )
 
         return max_points
 
@@ -906,6 +1014,9 @@ class ScanSegmentation:
         # self.anim.annotations.extend(anns)
         self.ann_biggest_ids.extend(biggest_ids)
 
+    def _prepare_clf_fn(self):
+        method = str(self.parameters.param("Segmentation Method").value())
+
     def run(self):
         logger.debug("run...")
         # GLCM
@@ -913,8 +1024,8 @@ class ScanSegmentation:
         # self._inner_texture.add_cols_to_report = False
 
         method = str(self.parameters.param("Segmentation Method").value())
-        if method == "HCTFS":
-            if bool(self.parameters.param("HCTFS", "Load Default Classifier").value()):
+        if method in ("HCTFS", "GLCMTFS"):
+            if bool(self.parameters.param("TFS General", "Load Default Classifier").value()):
                 if self.clf_default_fn.exists():
                     logger.debug(
                         f"Reading default classifier from {str(self.clf_default_fn)}"
@@ -923,16 +1034,18 @@ class ScanSegmentation:
                 else:
                     logger.error("Default classifier not found")
 
-        if bool(self.parameters.param("HCTFS", "Run Training").value()):
+        if bool(self.parameters.param("TFS General", "Run Training").value()):
             self.train_classifier()
             self.save_classifier()
+        if bool(self.parameters.param("Save Training Labels").value()):
+            self.save_training_labels()
         if bool(self.parameters.param("Run Prediction").value()):
             logger.debug("predict...")
             self.predict()
             logger.debug("evaluate...")
             self.evaluate()
-        if bool(self.parameters.param("Save Training Labels").value()):
-            self.save_training_labels()
+            self.evaluate_labels()
+            self._imsave_full_raster_and_training_labels()
 
         res = list(self.anim.get_pixel_size(self.level))
         logger.debug(f"slide segmentation resolution = {res}")
