@@ -158,7 +158,7 @@ class GLCMTextureMeasurement:
             {
                 "name": "Tile Spacing",
                 "type": "int",
-                # "value": 32,
+                # "value": 64,
                 "value": tile_spacing,
                 "suffix": "px",
             },
@@ -197,13 +197,15 @@ class GLCMTextureMeasurement:
         self.report = report
 
     def set_input_data(
-        self, view: image.View, annotation_id: int = None, lobulus_segmentation=None
+        self, view: image.View, annotation_id:int=None, lobulus_segmentation=None,
+            texture_label_str_id:str=None
     ):
         self.anim = view.anim
         self.annotation_id = annotation_id
         self.parent_view = view
         logger.trace(f"lobulus segmentation {lobulus_segmentation}")
         self.lobulus_segmentation = lobulus_segmentation
+        self.texture_label_str_id = texture_label_str_id
 
     def run(self, recalculate_view=True):
         """
@@ -233,12 +235,20 @@ class GLCMTextureMeasurement:
         texture_image = view.get_region_image(as_gray=True)
 
         fn_ann_id = "" if self.annotation_id is None else f"_{self.annotation_id}"
-        fn_id = "{}{}{}".format(self.filename_label, fn_ann_id, texture_label_fn)
+        tx_ann_id = "" if self.texture_label_str_id is None else f"_{self.texture_label_str_id}"
+        fn_id = "{}{}{}{}".format(self.filename_label, fn_ann_id, texture_label_fn, tx_ann_id)
         if self.report is not None:
             self.report.imsave(
-                "texture_input_image_{}.png".format(fn_id),
+                f"texture_input_image_mul255_{fn_id}.png",
                 (texture_image * 255).astype(np.uint8),
-                level=60 + self.report_severity_offset,
+                level=45 + self.report_severity_offset,
+                level_npz=40 + self.report_severity_offset,
+                level_skimage=20 + self.report_severity_offset,
+            )
+            self.report.imsave(
+                f"texture_input_image_mul1_{fn_id}.png",
+                texture_image,
+                level=45 + self.report_severity_offset,
                 level_npz=40 + self.report_severity_offset,
                 level_skimage=20 + self.report_severity_offset,
             )
@@ -277,8 +287,9 @@ class GLCMTextureMeasurement:
         plt.title("GLCM correlation")
         image.imshow_with_colorbar(energy[:, :, 2])
         mx = np.max(energy, axis=(0, 1))
-        # mn = np.min(energy, axis=(0, 1))
+        mn = np.min(energy, axis=(0, 1))
         logger.debug(f"GLCM max energy, homogeneity, correlation: {mx}")
+        logger.debug(f"GLCM min energy, homogeneity, correlation: {mn}")
         # plt.colorbar()
         if self.report is not None:
             self.report.savefig_and_show(
@@ -289,10 +300,16 @@ class GLCMTextureMeasurement:
         # if close_figs:
         #     plt.close(fig)
         # plt.savefig("glcm_features_{}.png".format(title))
-
+        logger.debug(f"glcm_features_{fn_id} saved")
         fig = plt.figure()
-        plt.imshow(energy)
+        # 0..1 normalization because energy (3rd channel) can be negative
+        w = np.ones([1, 1, 3])
+        w.ravel()[2] = 0.5
+        t = np.zeros([1, 1, 3])
+        t.ravel()[2] = 0.5
+        plt.imshow(energy * w + t)
         if self.report is not None:
+
             self.report.savefig_and_show(
                 "glcm_features_color_{}.png".format(fn_id),
                 fig,
@@ -323,13 +340,26 @@ class GLCMTextureMeasurement:
         #     "GLCM Homogenity": np.mean(e1[seg]),
         #     "GLCM Correlation": np.mean(e2[seg]),
         # }
+        if self.report:
+            self.report.imsave_as_fig(
+                f"glcm_texture_features_{fn_id}",
+                # (self.measured_features * 255).astype(np.uint8),
+                self.measured_features,
+                level=45 + self.report_severity_offset,
+                # level_npz=55 + self.report_severity_offset,
+                npz_level=40 + self.report_severity_offset,
+                # level_skimage=0,
+                # level_skimage=45 + self.report_severity_offset,
+            )
         if self.sni_predictor is not None:
             # texture processing is called twice
             self.sni_predictor.predict_area(row)
         if self.report is not None and self.add_cols_to_report:
             self.report.add_cols_to_actual_row(row)
+
         logger.debug(
-            f"GLCM textures for id {self.annotation_id if self.annotation_id is not None else '-'} finished"
+            f"GLCM textures for id={self.annotation_id if self.annotation_id is not None else '-'} "
+            f"(tx_label={tx_ann_id}) finished"
         )
 
         # plt.show()
